Documentation
¶
Overview ¶
sisimai (pronounced /ɕi.ɕi.ma.i/) is a library that decodes complex and diverse bounce emails and outputs the results of the delivery failure, such as the reason for the bounce and the recipient email address, in structured data. It is also possible to output in JSON format. More information are available at https://libsisimai.org
Index ¶
- func Args() *siba.DecodingArgs
- func Dump(path string, args *siba.DecodingArgs) (*string, []siba.NotDecoded)
- func Factor() *siba.Fact
- func Reason() map[string]string
- func Rise(path string, args *siba.DecodingArgs) ([]siba.Fact, []siba.NotDecoded)
- func Version() string
- type CallbackArg0
- type CallbackArg1
- type CfParameter0
- type CfParameter1
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func Args ¶
func Args() *siba.DecodingArgs
Args returns a pointer to siba.DecodingArgs as the 2nd argument of the Rise function.
func Dump ¶
func Dump(path string, args *siba.DecodingArgs) (*string, []siba.NotDecoded)
Dump returns decoded data as a JSON string.
Arguments: - path (string): Path to an mbox, Maildir/, or "STDIN" for standard input. - args (*siba.DecodingArgs): Options and callback functions for decoding bounce messages. Returns: - (*string): Decoded data as a JSON string array - ([]siba.NotDecoded): List of occurred errors
func Rise ¶
func Rise(path string, args *siba.DecodingArgs) ([]siba.Fact, []siba.NotDecoded)
Rise is a function for decoding bounce mails in a mailbox or a Maildir/.
Arguments: - path (string): Path to an UNIX mbox, Maildir/, or "STDIN" for standard input. - args (*siba.DecodingArgs): Options and callback functions for decoding bounce messages. Returns: - ([]siba.Fact): List of successfully decoded bounce messages. - ([]siba.NotDecoded): List of occurred errors.
Types ¶
type CallbackArg0 ¶
type CallbackArg0 = siba.CallbackArg0
type CallbackArg1 ¶
type CallbackArg1 = siba.CallbackArg1
type CfParameter0 ¶
type CfParameter0 = siba.CfParameter0
type CfParameter1 ¶
type CfParameter1 = siba.CfParameter1
Directories
¶
| Path | Synopsis |
|---|---|
|
Package "address" provide functions related to an email address.
|
Package "address" provide functions related to an email address. |
|
Package "arf" provides ARF: Abuse Report Format related functions.
|
Package "arf" provides ARF: Abuse Report Format related functions. |
|
Package "eb" provides constants for the email bounce.
|
Package "eb" provides constants for the email bounce. |
|
Package "fact" provide a function for generating structs keeping decoded bounce mail data.
|
Package "fact" provide a function for generating structs keeping decoded bounce mail data. |
|
Package "lda" provides a function to detect a bounce reason due to errors returned from LDA.
|
Package "lda" provides a function to detect a bounce reason due to errors returned from LDA. |
|
Package "lhost" provides functions for decoding bounce messages generated by various MTAs such as Sendmail, Postfix, OpenSMTPD, some commercial mail servers, and other email services.
|
Package "lhost" provides functions for decoding bounce messages generated by various MTAs such as Sendmail, Postfix, OpenSMTPD, some commercial mail servers, and other email services. |
|
Package "mail" provides funtions for reading a UNIX mbox, a Maildir, or any email message input from Standard-in.
|
Package "mail" provides funtions for reading a UNIX mbox, a Maildir, or any email message input from Standard-in. |
|
Package "message" provides functions to read email message as a string, to tidy up each line.
|
Package "message" provides functions to read email message as a string, to tidy up each line. |
|
Package "moji" provides functions for dealing strings.
|
Package "moji" provides functions for dealing strings. |
|
Package "reason" provides functions for detecting the bounce reason by matching many error message patterns defined in why-*.go files.
|
Package "reason" provides functions for detecting the bounce reason by matching many error message patterns defined in why-*.go files. |
|
package "rfc1123" provides functions related to Internet hosts described in RFC1123.
|
package "rfc1123" provides functions related to Internet hosts described in RFC1123. |
|
Package "rfc1894" provides funtions related to RFC1894; An Extensible Message Format for Delivery Status Notifications.
|
Package "rfc1894" provides funtions related to RFC1894; An Extensible Message Format for Delivery Status Notifications. |
|
Package "rfc2045" provides functions for RFC2045; Multipurpose Internet Mail Extensions (MIME) Part One: Format of Internet Message Bodies.
|
Package "rfc2045" provides functions for RFC2045; Multipurpose Internet Mail Extensions (MIME) Part One: Format of Internet Message Bodies. |
|
Package "rfc3464" provides functions like a MTA module in "lhost" package for decoding bounce messages formatted according to RFC3464; An Extensible Message Format for Delivery Status Notifications.
|
Package "rfc3464" provides functions like a MTA module in "lhost" package for decoding bounce messages formatted according to RFC3464; An Extensible Message Format for Delivery Status Notifications. |
|
Package "rfc3834" provides functions like a MTA module in "lhost" package for decoding automatic responded messages formatted according to RFC3834; Recommendations for Automatic Responses to Electronic Mail.
|
Package "rfc3834" provides functions like a MTA module in "lhost" package for decoding automatic responded messages formatted according to RFC3834; Recommendations for Automatic Responses to Electronic Mail. |
|
Package "rfc5322" provides functions for email addresses, Date: header, Received: headers, and other headers and messages related to RFC5322.
|
Package "rfc5322" provides functions for email addresses, Date: header, Received: headers, and other headers and messages related to RFC5322. |
|
Package "rfc5965" provides a function related to RFC5965; An Extensible Format for Email Feedback Reports.
|
Package "rfc5965" provides a function related to RFC5965; An Extensible Format for Email Feedback Reports. |
|
Package "rfc791" provides functions related to IPv4 address described in RFC791.
|
Package "rfc791" provides functions related to IPv4 address described in RFC791. |
|
Package "rhost" provides functions for detecting bounce reason by matching error codes or error error message patterns described in documents of each email services.
|
Package "rhost" provides functions for detecting bounce reason by matching error codes or error error message patterns described in documents of each email services. |
|
Package "siba" provides types for data structure of sisimai SIBA stands for Sisimai Internal Bounce Abstraction.
|
Package "siba" provides types for data structure of sisimai SIBA stands for Sisimai Internal Bounce Abstraction. |
|
smtp
|
|
|
command
Package "smtp/command" provides functions related to SMTP commands.
|
Package "smtp/command" provides functions related to SMTP commands. |
|
failure
Package "smtp/failure" provides functions related to SMTP errors.
|
Package "smtp/failure" provides functions related to SMTP errors. |
|
reply
Package "smtp/reply" provides funtions related to SMTP reply codes such as 421, 550.
|
Package "smtp/reply" provides funtions related to SMTP reply codes such as 421, 550. |
|
status
Package "smtp/status" provides functions related to SMTP Status codes such as 4.2.2, 5.1.1.
|
Package "smtp/status" provides functions related to SMTP Status codes such as 4.2.2, 5.1.1. |
|
transcript
Package "smtp/transcript" provides functions related to SMTP transcript logs.
|
Package "smtp/transcript" provides functions related to SMTP transcript logs. |
Click to show internal directories.
Click to hide internal directories.